# -*- coding: utf-8 -*-
import sys
import requests

from suite_py.lib.handler.captainhook_handler import CaptainHook
from suite_py.lib.logger import Logger

logger = Logger()


def entrypoint(project, timeout, env, action):
    captainhook = CaptainHook()
    if timeout:
        captainhook.set_timeout(timeout)
    env = parse_env(env)
    if action == "lock":
        try:
            req = captainhook.lock_project(project, env)
            handle_request(req)
            logger.info(f"Bloccato deploy su {env} del progetto {project}")
        except requests.exceptions.Timeout:
            logger.warning(
                "Richiesta a Captainhook in timeout. Prova con suite-py --timeout=60 lock-project lock"
            )
            sys.exit(1)
    elif action == "unlock":
        try:
            req = captainhook.unlock_project(project, env)
            handle_request(req)
            logger.info(f"Abilitato deploy su {env} del progetto {project}")
        except requests.exceptions.Timeout:
            logger.warning(
                "Richiesta a Captainhook in timeout. Prova con suite-py --timeout=60 lock-project unlock"
            )
            sys.exit(1)
    else:
        logger.warning("Non ho capito che cosa devo fare")
        sys.exit(-1)


def handle_request(request):
    if request.status_code != 200:
        logger.error(
            "Qualcosa è andato storto durante la richiesta. Richiedi supporto ai devops su slack."
        )
        sys.exit(-1)

    return True


def parse_env(env):
    # implementare uno switch era troppo noioso
    if env == "deploy":
        return "production"
    if env == "merge":
        return "staging"
    return env



#! -*- encoding: utf-8 -*-
import os
import sys

import click
from autoupgrade import Package

from suite_py.commands import merge_pr
from suite_py.commands import deploy
from suite_py.commands import rollback
from suite_py.commands import create_branch
from suite_py.commands import open_pr
from suite_py.commands import project_lock
from suite_py.commands import ask_review
from suite_py.commands import create_qa
from suite_py.commands import delete_qa
from suite_py.commands import status
from suite_py.commands import check
from suite_py.commands import ip
from suite_py.commands import generator
from suite_py.lib.handler import prompt_utils
from suite_py.lib.config import Config
from suite_py.__version__ import __version__


config = Config()

# TODO: fixme -> write something more smart in locating the current project
def confirm_cwd():
    project_path = os.getcwd()
    project = os.path.basename(project_path)
    if not prompt_utils.ask_confirm(f"Vuoi continuare sul progetto {project}?"):
        sys.exit(-1)
    return project_path


@click.group()
@click.option(
    "--project",
    type=click.Path(exists=True),
    default=confirm_cwd,
    help="Nome del progetto su cui eseguire il comando (default directory corrente)",
)
@click.option(
    "--timeout",
    type=click.INT,
    help="Timeout in secondi per le operazioni di CaptainHook",
)
@click.pass_context
def main(ctx, project, timeout):
    Package("suite_py", verbose=True).smartupgrade()
    print(f"v{__version__}")
    project_name = os.path.basename(project)
    ctx.ensure_object(dict)
    ctx.obj["project"] = project_name
    ctx.obj["captainhook_timeout"] = timeout  # Might be also None
    os.chdir(os.path.join(config.load()["user"]["projects_home"], ctx.obj["project"]))


@main.command(
    "create-branch", help="Crea branch locale e imposta la card di YouTrack in progress"
)
@click.option("--card", type=click.STRING, help="Numero card youtrack (ex. PRIMA-123)")
@click.pass_context
def cli_create_branch(ctx, card):
    create_branch.entrypoint(ctx.obj["project"], card)


@main.command("lock", help="Lock di un progetto su staging o prod")
@click.argument(
    "environment", type=click.Choice(("staging", "production", "deploy", "merge"))
)
@click.pass_context
def cli_lock_project(ctx, environment):
    project_lock.entrypoint(
        ctx.obj["project"], ctx.obj["captainhook_timeout"], environment, "lock"
    )


@main.command("unlock", help="Unlock di un progetto su staging o prod")
@click.argument(
    "environment", type=click.Choice(("staging", "production", "deploy", "merge"))
)
@click.pass_context
def cli_unlock_project(ctx, environment):
    project_lock.entrypoint(
        ctx.obj["project"], ctx.obj["captainhook_timeout"], environment, "unlock"
    )


@main.command("open-pr", help="Apre una PR su GitHub")
@click.pass_context
def cli_open_pr(ctx):
    open_pr.entrypoint(ctx.obj["project"])


@main.command("ask-review", help="Chiede la review di una PR")
@click.pass_context
def cli_ask_review(ctx):
    ask_review.entrypoint(ctx.obj["project"], ctx.obj["captainhook_timeout"])


@main.command("create-qa", help="Crea un QA (integrazione con qainit)")
@click.pass_context
def cli_create_qa(ctx):
    create_qa.entrypoint(ctx.obj["project"])


@main.command("delete-qa", help="Cancella un QA (integrazione con qainit)")
@click.pass_context
def cli_delete_qa(ctx):
    delete_qa.entrypoint(ctx.obj["project"])


@main.command(
    "merge-pr", help="Merge del branch selezionato con master se tutti i check sono ok"
)
@click.pass_context
def cli_merge_pr(ctx):
    merge_pr.entrypoint(ctx.obj["project"], ctx.obj["captainhook_timeout"])


@main.command("deploy", help="Deploy in produzione del branch master")
@click.pass_context
def cli_deploy(ctx):
    deploy.entrypoint(ctx.obj["project"], ctx.obj["captainhook_timeout"])


@main.command("rollback", help="Rollback in produzione")
@click.pass_context
def cli_rollback(ctx):
    rollback.entrypoint(ctx.obj["project"], ctx.obj["captainhook_timeout"])


@main.command("status", help="Stato attuale di un progetto")
@click.pass_context
def cli_status(ctx):
    status.entrypoint(ctx.obj["project"], ctx.obj["captainhook_timeout"])


@main.command("check", help="Verifica autorizzazioni ai servizi di terze parti")
@click.pass_context
def cli_check(ctx):
    check.entrypoint(ctx.obj["captainhook_timeout"])


@main.command(
    "ip", help="Ottieni gli indirizzi IP degli hosts dove il task è running",
)
@click.argument("environment", type=click.Choice(("staging", "production")))
@click.pass_context
def cli_ip(ctx, environment):
    ip.entrypoint(ctx.obj["project"], environment)


@main.command("generator", help="Genera diversi file partendo da dei templates")
def cli_generator():
    generator.entrypoint()



# -*- coding: utf-8 -*-
import sys
import semver

from halo import Halo

from suite_py.lib.handler.youtrack_handler import YoutrackHandler
from suite_py.lib.handler.github_handler import GithubHandler
from suite_py.lib.handler.captainhook_handler import CaptainHook
from suite_py.lib.handler import prompt_utils
from suite_py.lib.handler import git_handler as git
from suite_py.lib.handler import drone_handler as drone
from suite_py.lib.logger import Logger
from suite_py.lib.config import Config


youtrack = YoutrackHandler()
github = GithubHandler()
config = Config()
logger = Logger()


def entrypoint(project, timeout):

    lock = CaptainHook()
    if timeout:
        lock.set_timeout(timeout)
    stop_if_prod_locked(project, lock)

    git.fetch(project)
    repo = github.get_repo(project)

    latest_release = get_release(repo)

    versions = bump_versions(latest_release.title)

    commits = github.get_commits_since_release(repo, latest_release)

    check_migrations_deploy(commits)

    message = "\n".join(["* " + c.commit.message.splitlines()[0] for c in commits])

    logger.info(f"\nLista dei commit:\n{message}\n")

    if not prompt_utils.ask_confirm("Vuoi continuare?"):
        sys.exit(-1)

    new_version = prompt_utils.ask_choices(
        "Seleziona versione:",
        [
            {"name": f"Patch {versions['patch']}", "value": versions["patch"]},
            {"name": f"Minor {versions['minor']}", "value": versions["minor"]},
            {"name": f"Major {versions['major']}", "value": versions["major"]},
        ],
    )

    release_state = config.load()["youtrack"]["release_state"]

    issue_ids = youtrack.get_issue_ids(commits)

    if len(issue_ids) > 0:
        update_youtrack_state = prompt_utils.ask_confirm(
            f"Vuoi spostare le card associate in {release_state}?", default=False
        )

        for issue_id in issue_ids:
            try:
                youtrack.comment(
                    issue_id,
                    f"Deploy in produzione di {project} effettuato con la release {new_version}",
                )
                if update_youtrack_state:
                    youtrack.update_state(issue_id, release_state)
                    logger.info(f"{issue_id} spostata in {release_state}")
            except Exception:
                logger.warning(
                    f"Si è verificato un errore durante lo spostamento della card {issue_id} in {release_state}"
                )

    create_release(repo, new_version, message, project)


def get_release(repo):
    with Halo(text="Loading...", spinner="dots", color="magenta"):
        latest_release = github.get_latest_release_if_exists(repo)
        tags = repo.get_tags()
        tag = git.get_last_tag_number(tags)
    if latest_release and latest_release.title == tag:
        logger.info(f"La release attuale è {latest_release.title}")
        return latest_release
    logger.info(f"L'ultimo tag trovato è {tag}")
    if not prompt_utils.ask_confirm("Sicuro di voler continuare?"):
        sys.exit(-1)
    return repo.create_git_release(tag, tag, f"New release from tag {tag}")


def create_release(repo, new_version, message, project):
    new_release = repo.create_git_release(
        new_version, new_version, youtrack.replace_card_names_with_md_links(message)
    )
    if new_release:
        logger.info(f"La release è stata creata! Link: {new_release.html_url}")

        drone_url = drone.get_last_build_url(project)
        if drone_url:
            logger.info(f"Puoi seguire il deploy in produzione su {drone_url}")


def bump_versions(current):
    return {
        "patch": semver.bump_patch(current),
        "minor": semver.bump_minor(current),
        "major": semver.bump_major(current),
    }


def deployed_cards_to_string(cards):
    if cards:
        return "\n".join(cards)
    return ""


def check_migrations_deploy(commits):
    if not commits:
        logger.error("ERRORE: nessun commit trovato")
        sys.exit(-1)
    elif len(commits) == 1:
        files_changed = git.files_changed_between_commits("--raw", f"{commits[0].sha}~")
    else:
        files_changed = git.files_changed_between_commits(
            f"{commits[-1].sha}~", commits[0].sha
        )
    if git.migrations_found(files_changed):
        logger.warning("ATTENZIONE: migrations rilevate nel codice")
        if not prompt_utils.ask_confirm("Sicuro di voler continuare?"):
            sys.exit(-1)


def stop_if_prod_locked(project, lock):
    request = lock.status(project, "production")

    if request.status_code != 200:
        logger.error("Impossibile determinare lo stato del lock su master.")
        sys.exit(-1)

    request_object = request.json()
    if request_object["locked"]:
        logger.error(
            f"Il progetto è lockato in produzione da {request_object['by']}. Impossibile continuare."
        )
        sys.exit(-1)



# -*- encoding: utf-8 -*-
from termcolor import colored

from suite_py.lib.singleton import Singleton


class Logger(metaclass=Singleton):
    def __init__(self):
        pass

    def info(self, message):
        print(colored(message, "green"))

    def warning(self, message):
        print(colored(message, "yellow"))

    def error(self, message):
        print(colored(message, "red"))



# -*- coding: utf-8 -*-
from halo import Halo

from suite_py.lib.handler.captainhook_handler import CaptainHook
from suite_py.lib.config import Config
from suite_py.lib.logger import Logger

logger = Logger()
config = Config()

CHECKMARK = "✔"
CROSSMARK = "✘"


def entrypoint(project, timeout):
    captainhook = CaptainHook()
    if timeout:
        captainhook.set_timeout(timeout)

    with Halo(text="Contacting Captainhook...", spinner="dots", color="magenta"):
        staging_status = captainhook.status(project, "staging").json()
        production_status = captainhook.status(project, "production").json()

    forge_message(staging_status, "staging")
    forge_message(production_status, "production")


def forge_message(status, env):
    if status["locked"]:
        if status["by"] == "":
            logger.error(f"{CROSSMARK} {env}\n      lockato")
        else:
            logger.error(f"{CROSSMARK} {env}\n      lockato da {status['by']}")
    else:
        logger.info(f"{CHECKMARK} {env}\n      non lockato")



# -*- coding: utf-8 -*-
import os
from os import listdir
from os.path import isfile, join
import stat
import subprocess
from datetime import datetime
import yaml
from jinja2 import Environment, DebugUndefined, FileSystemLoader
from suite_py.lib.handler.github_handler import GithubHandler
from suite_py.lib.handler import git_handler as git
from suite_py.lib.handler import prompt_utils
from suite_py.lib.logger import Logger
from suite_py.lib.config import Config
from suite_py.lib.handler import drone_handler as drone

github = GithubHandler()
logger = Logger()
config = Config()

now = datetime.now().strftime("%Y%m%d%H%M")

# define paths
projects_root = config.load()["user"]["projects_home"]
artemide_path = os.path.join(projects_root, "artemide")
drone_values_path = f"{artemide_path}/templates/drone/values/"
deploy_scripts_values_path = f"{artemide_path}/templates/deploy_scripts/values/"


def entrypoint():
    # move to artemide root directory
    os.chdir(artemide_path)

    env = Environment(
        loader=FileSystemLoader(artemide_path),
        undefined=DebugUndefined,
        trim_blocks=True,
        lstrip_blocks=True,
    )

    template_type = prompt_utils.ask_choices(
        "Quali file vuoi generare?", ["drone", "deploy_scripts"]
    )
    message = f"Autogenerated update {template_type} from suite-py {now}"

    autobranch = prompt_utils.ask_confirm(
        "Vuoi creare branch e pr in automatico?", default=False
    )

    if autobranch:
        autopush = True
    else:
        autopush = prompt_utils.ask_confirm(
            "Vuoi pushare in automatico?", default=False
        )

    choice = prompt_utils.ask_questions_input(
        "Su quale progetto vuoi lavorare? Inserisci `all` per selezionarli tutti: ",
        "all",
    )

    if choice == "all":
        projects = get_all_templated_projects(template_type)
    else:
        projects = [choice]

    for project in projects:
        logger.info(f"{project}: Controllo che non ci siano file non committati")
        if git.is_dirty(project):
            logger.error(
                f"{project}: Ci sono modifiche non committate, skippo il progetto"
            )
            continue

        project_root_path = os.path.join(projects_root, project)
        branch_name = configure_branch(project, template_type, autobranch, autopush)

        logger.info(f"{project}: Generazione in corso")
        # launch generate
        if template_type == "drone":
            generate_drone(project_root_path, f"{drone_values_path}{project}.yml", env)
        elif template_type == "deploy_scripts":
            generate_deploy_scripts(
                project_root_path, f"{deploy_scripts_values_path}{project}.yml", env
            )
        logger.info(f"{project}: Generazione completata")

        if autobranch or autopush:
            # controllo che sia cambiato effettivamente qualcosa
            if git.is_dirty(project):
                git_operations(project, autobranch, autopush, branch_name, message)
            else:
                logger.warning(
                    f"{project}: nessuna operazione su git da fare. Nessun file modificato"
                )


def get_all_templated_projects(template_type):
    if template_type == "drone":
        values_path = drone_values_path
    elif template_type == "deploy_scripts":
        values_path = deploy_scripts_values_path

    return [
        f.replace(".yml", "")
        for f in listdir(values_path)
        if isfile(join(values_path, f))
    ]


def configure_branch(project, template_type, autobranch, autopush):
    logger.info(f"{project}: Sincronizzo il repo con il remote")
    if autobranch:
        git.sync(project)  # fa anche checkout su master
        return f"t/autogenerated_update_{template_type}_{now}"

    current_branch = git.current_branch_name(project)
    if autopush and git.remote_branch_exists(project, current_branch):
        git.fetch(project)
        git.pull(project)

    return current_branch


def generate_drone(project_path, values_file, env):
    values = yaml.safe_load(open(values_file))
    template = env.get_template(f"templates/drone/{values['template']}.j2")
    rendered = template.render(values)

    write_on_repo(project_path, rendered, ".drone.yml")
    drone.fmt(values["project_name"], project_path)
    drone.validate(values["project_name"], project_path)
    drone.sign(values["project_name"], project_path)


def generate_deploy_scripts(project_path, values_file, env):
    values = yaml.safe_load(open(values_file))
    template = env.get_template(f"templates/deploy_scripts/{values['template']}.j2")

    rendered = template.render(values)
    file_name = "deploy"
    write_on_repo(os.path.join(project_path, "deploy"), rendered, file_name)
    validate_deploy_script(f"{project_path}/deploy/{file_name}")

    if values["cloudformation"]:
        template = env.get_template("templates/deploy_scripts/wait_for_stack.j2")
        rendered = template.render(values)
        file_name = "wait_for_stack"
        write_on_repo(os.path.join(project_path, "deploy"), rendered, file_name)
        validate_deploy_script(f"{project_path}/deploy/{file_name}")


def write_on_repo(path, rendered, file_name):
    if not os.path.exists(path):
        os.mkdir(path, mode=0o755)

    fd = open(os.path.join(path, file_name), "w")
    fd.write(rendered)
    fd.close()


def make_executable(filepath):
    st = os.stat(filepath)
    os.chmod(filepath, st.st_mode | stat.S_IEXEC)


def validate_deploy_script(filepath):
    make_executable(filepath)
    subprocess.run(
        ["shellcheck", filepath], check=True,
    )


def git_operations(project, autobranch, autopush, branch_name, message):
    if autobranch:
        logger.info(f"{project}: creo il branch {branch_name}")
        git.checkout(project, branch_name, new=True)
        git.add(project)
        git.commit(project, message)
        git.push(project, branch_name)
        pr = github.create_pr(project, branch_name, message)
        logger.info(f"Pull request numero {pr.number} creata! {pr.html_url}")
    elif autopush:
        if git.current_branch_name(project) != "master":
            git.add(project)
            git.commit(project, message)
            git.push(project, branch_name)
        else:
            logger.warning(
                f"{project} si trova su master, non eseguo il push automatico"
            )



# -*- coding: utf-8 -*-
import readline
import functools
import sys

from suite_py.lib.config import Config
from suite_py.lib.handler.youtrack_handler import YoutrackHandler
from suite_py.lib.handler.github_handler import GithubHandler
from suite_py.lib.logger import Logger
from suite_py.lib.handler import git_handler as git
from suite_py.lib.handler.captainhook_handler import CaptainHook

youtrack = YoutrackHandler()
github = GithubHandler()
logger = Logger()
config = Config()


def maybe_get_users_list(timeout):
    try:
        captainhook = CaptainHook()
        if timeout:
            captainhook.set_timeout(timeout)
        users = captainhook.get_users_list().json()
        config.put_cache("users", users)
        return users
    except Exception:
        logger.warning(
            "Non riesco ad ottenere la lista degli utenti da captainhook. Utilizzo la versione in cache."
        )
        return config.get_cache("users")


def get_pr(project):
    branch_name = git.current_branch_name(project)
    pull = github.get_pr_from_branch(project, branch_name)

    if pull.totalCount:
        pr = pull[0]
        logger.info(
            f"Ho trovato la pull request numero {pr.number} per il branch {branch_name} sul repo {project}"
        )
    else:
        logger.error(f"Nessuna pull request aperta trovata per il branch {branch_name}")
        sys.exit(-1)

    return pr


def ask_reviewer(users):
    u_completer = functools.partial(completer, users)
    readline.set_completer(u_completer)
    readline.parse_and_bind("tab: complete")

    youtrack_reviewers = []

    youtrack_reviewers = list(
        input(
            "Scegli i reviewers (nome.cognome - separati da spazio - premere TAB per autocomplete) > "
        ).split()
    )

    if not youtrack_reviewers:
        logger.warning("Devi inserire almeno un reviewer")
        return ask_reviewer(users)

    return youtrack_reviewers


def completer(users, text, state):
    options = [x["youtrack"] for x in users if text.lower() in x["youtrack"].lower()]
    try:
        return options[state]
    except IndexError:
        return None


def find_github_nicks(youtrack_reviewers, users):
    github_reviewers = []
    for rev in youtrack_reviewers:
        for user in users:
            if user["youtrack"] == rev:
                github_reviewers.append(user["github"])

    return github_reviewers


def maybe_adjust_youtrack_card(title, youtrack_reviewers):
    youtrack_id = youtrack.get_card_from_name(title)

    if youtrack_id:
        logger.info(
            f"Sposto la card {youtrack_id} in review su youtrack e aggiungo i tag degli utenti"
        )
        youtrack.update_state(youtrack_id, "Review")
        for rev in youtrack_reviewers:
            try:
                youtrack.add_tag(youtrack_id, f"review:{rev}")
            except BaseException as e:
                logger.warning(f"Non sono riuscito ad aggiungere i tag di review: {e}")
                sys.exit(-1)
    else:
        logger.warning(
            "Reviewers inseriti SOLO su GitHub. Nessuna card collegata o card nel nome del branch inesistente su YouTrack."
        )


def entrypoint(project, timeout):
    users = maybe_get_users_list(timeout)
    pr = get_pr(project)
    youtrack_reviewers = ask_reviewer(users)
    github_reviewers = find_github_nicks(youtrack_reviewers, users)
    pr.create_review_request(github_reviewers)
    logger.info("Aggiungo reviewers su GitHub")
    maybe_adjust_youtrack_card(pr.title, youtrack_reviewers)



# -*- coding: utf-8 -*-
import sys

from halo import Halo

from suite_py.lib.handler.youtrack_handler import YoutrackHandler
from suite_py.lib.handler.github_handler import GithubHandler
from suite_py.lib.handler.captainhook_handler import CaptainHook
from suite_py.lib.config import Config
from suite_py.lib.logger import Logger
from suite_py.lib.handler import prompt_utils
from suite_py.lib.qainit import qainit_shutdown
from suite_py.lib.handler import git_handler as git
from suite_py.lib.handler import drone_handler as drone


youtrack = YoutrackHandler()
github = GithubHandler()
logger = Logger()
config = Config()

CHECKMARK = "✔"
CROSSMARK = "✘"


def entrypoint(project, timeout):
    captainhook = CaptainHook()
    if timeout:
        captainhook.set_timeout(timeout)

    stop_if_master_locked(project, captainhook)

    pr = select_pr(project)

    print(f"\nHai selezionato: \n{check_pr_status(project, pr)}")
    if not prompt_utils.ask_confirm("Vuoi continuare il merge?", default=False):
        sys.exit(0)

    branch_name = pr.head.ref
    youtrack_id = youtrack.get_card_from_name(branch_name)

    check_migrations_merge(pr)

    logger.info("Eseguo il merge...")

    merge_status = pr.merge(
        commit_title=f"{pr.title} (#{pr.number})",
        commit_message="",
        merge_method="squash",
    )

    if not merge_status.merged:
        logger.error("Si è verificato un errore durante il merge.")
        sys.exit(-1)

    drone_build_number = drone.get_pr_build_number(project, merge_status.sha)
    drone_build_url = drone.get_build_url(project, drone_build_number)

    if drone_build_url:
        logger.info(
            f"Pull request mergiata su master! Puoi seguire lo stato della build su {drone_build_url}"
        )
    else:
        logger.info("Pull request mergiata su master!")

    git.fetch(project)
    if git.remote_branch_exists(project, branch_name):
        git.delete_remote_branch(project, branch_name)

    if prompt_utils.ask_confirm(
        "Vuoi bloccare staging? (Necessario se bisogna testare su staging)",
        default=False,
    ):
        if drone.prestart_success(project, drone_build_number):
            captainhook.lock_project(project, "staging")
        else:
            logger.error(
                "Problemi con la build su drone, non riesco a bloccare staging"
            )
            sys.exit(-1)

    if youtrack_id:
        logger.info("Aggiorno lo stato della card su youtrack...")
        youtrack.update_state(youtrack_id, config.load()["youtrack"]["merged_state"])
        logger.info("Card aggiornata")

        logger.info("Spengo il qa, se esiste")
        qainit_shutdown(youtrack_id)
    else:
        logger.warning(
            "Non sono riuscito a trovare una issue YouTrack nel nome del branch o la issue indicata non esiste."
        )
        logger.warning(
            "Nessuna card aggiornata su YouTrack e nessun QA spento in automatico"
        )

    logger.info("Tutto fatto!")
    sys.exit()


def select_pr(project):
    if github.user_is_admin(project):
        logger.warning(
            "Sei admin del repository, puoi fare il merge skippando i check (CI, review, ecc...)\nDa grandi poteri derivano grandi responsabilita'"
        )

    with Halo(text="Loading pull requests...", spinner="dots", color="magenta"):
        choices = [
            {"name": pr.title, "value": pr} for pr in github.get_list_pr(project)
        ]
    if choices:
        choices.sort(key=lambda x: x["name"])
        return prompt_utils.ask_choices("Seleziona PR: ", choices)

    logger.error(
        f"Non esistono pull request pronte per il merge o potresti non avere i permessi, per favore controlla su https://github.com/primait/{project}/pulls"
    )
    sys.exit(-1)


def stop_if_master_locked(project, captainhook):
    request = captainhook.status(project, "staging")

    if request.status_code != 200:
        logger.error("Impossibile determinare lo stato del lock su master.")
        sys.exit(-1)

    request_object = request.json()
    if request_object["locked"]:
        logger.error(
            f"Il progetto è lockato su staging da {request_object['by']}. Impossibile continuare."
        )
        sys.exit(-1)


def check_migrations_merge(pr):
    files_changed = [x.filename for x in pr.get_files()]
    if git.migrations_found(files_changed):
        logger.warning("ATTENZIONE: migrations rilevate nel codice")
        if not prompt_utils.ask_confirm("Sicuro di voler continuare?"):
            sys.exit(-1)


def check_pr_status(project, pr):
    build_status = CHECKMARK
    reviews = CHECKMARK
    if pr.mergeable_state == "blocked":
        build_status = pr_build_status(project, pr)
        reviews = pr_reviews(pr)

    return f"#{pr.number} {pr.title}\n     build: {build_status} - reviews: {reviews}\n"


def pr_build_status(project, pr):
    if github.get_build_status(project, pr.head.sha).state == "success":
        return CHECKMARK
    return CROSSMARK


def pr_reviews(pr):
    reviews = [r for r in pr.get_reviews() if r.state == "APPROVED"]
    if reviews:
        return CHECKMARK
    return CROSSMARK
